@@ -2,7 +2,7 @@ from collections import Counter
 import datetime
 import decimal
 from pathlib import Path
-from typing import Dict, List, Optional, Union
+from typing import List, Optional, Union
 
 from jinja2 import Environment, FileSystemLoader, StrictUndefined
 from pydantic import BaseModel, validator, Extra
@@ -12,8 +12,6 @@ import sqlalchemy
 
 from ..core.errors import DBError, Exc, Ok
 
-from ..utils.misc import group_list
-
 
 class Hook(BaseModel):
     sql: str
@@ -21,72 +19,40 @@ class Hook(BaseModel):
 
 class Columns(BaseModel):
     name: str
-    type: Optional[str] = None
-    dst_name: Optional[str] = None
+    type: Optional[str]
+    dst_name: Optional[str]
     description: Optional[str]
 
     class Tests(BaseModel):
         name: Optional[str]
-        values: Optional[List[str]]
+        allowed_values: Optional[List[str]]
         execute: bool = True
 
         class Config:
             extra = Extra.forbid
 
-    tests: Optional[List[Union[str, Tests]]] = list()
+    tests: List[Union[str, Tests]] = list()
 
     class Config:
         extra = Extra.forbid
 
 
 class DDL(BaseModel):
-    class Properties(BaseModel):
-        class Indexes(BaseModel):
-            columns: Optional[List[Union[str, Dict]]]
-
-        indexes: Optional[Dict[str, Indexes]]
-
-        class Config:
-            extra = Extra.forbid
-
-        @validator("indexes")
-        def index_columns_exists(cls, v, values):
-            cols = [c.name for c in values.get("columns", list())]
-            if len(cols) > 0:
-                missing_cols = group_list(
-                    [
-                        (index_name, index_column)
-                        for index_name, index in v.items()
-                        for index_column in index.columns
-                        if index_column not in cols
-                    ]
-                )
-                if len(missing_cols) > 0:
-                    cols_msg = ";".join(
-                        [f"On {i}: {','.join(c)}" for i, c in missing_cols.items()]
-                    )
-                    raise ValueError(
-                        f"Some indexes refer to missing columns: {cols_msg}"
-                    )
-
-            return v
-
-    columns: Optional[List[Columns]] = list()
-    properties: Optional[List[Properties]] = list()
-    post_hook: Optional[List[Hook]] = list()
+    columns: List[Union[str, Columns]] = list()
+    post_hook: List[Hook] = list()
 
     class Config:
         extra = Extra.forbid
 
     @validator("columns", pre=True)
-    def transform_str_cols(cls, v, values):
+    def transform_str_cols(cls, v):
         if v is not None and isinstance(v, List):
             return [{"name": c} if isinstance(c, str) else c for c in v]
         else:
             return v
 
     @validator("columns")
-    def columns_unique(cls, v, values):
+    def columns_unique(cls, v):
         dupes = {k for k, v in Counter([e.name for e in v]).items() if v > 1}
         if len(dupes) > 0:
             raise ValueError(f"Duplicate columns: {','.join(dupes)}")
@@ -95,21 +61,24 @@ class DDL(BaseModel):
 
     def get_ddl(self):
         cols = self.columns
-        self.columns = []
+
+        columns = list()
         for c in cols:
-            tests = []
+            tests = list()
             for t in c.tests:
                 if isinstance(t, str):
-                    tests.append({"type": t, "values": [], "execute": True})
+                    tests.append({"type": t, "allowed_values": [], "execute": True})
                 else:
                     tests.append(
                         {
-                            "type": t.name if t.name is not None else "values",
-                            "values": t.values if t.values is not None else [],
+                            "type": t.name if t.name is not None else "allowed_values",
+                            "allowed_values": t.allowed_values
+                            if t.allowed_values is not None
+                            else [],
                             "execute": t.execute,
                         }
                     )
-            self.columns.append(
+            columns.append(
                 {
                     "name": c.name,
                     "description": c.description,
@@ -119,118 +88,12 @@ class DDL(BaseModel):
                 }
             )
 
-        props = self.properties
-        self.properties = []
-        for p in props:
-            self.properties.append(p.dict())
-
-        hook = self.post_hook
-        self.post_hook = []
-        for h in hook:
-            self.post_hook.append(h.dict())
-
-        res = {
-            "columns": self.columns,
-            "properties": self.properties,
-            "post_hook": self.post_hook,
+        return {
+            "columns": columns,
+            "properties": list(),
+            "post_hook": [h.dict() for h in self.post_hook],
         }
 
-        return res
-
-
-class DbObject:
-    def __init__(
-        self, connection_name, database, schema, object, stringify, prod_stringify
-    ):
-        self.connection_name = connection_name
-        self.database = database
-        self.schema = schema
-        self.object = object
-        self.stringify = stringify
-        self.prod_stringify = prod_stringify
-
-        self.database_value = None
-        self.database_prod_value = None
-        self.schema_value = None
-        self.schema_prod_value = None
-        self.object_value = None
-        self.object_prod_value = None
-        self.value = ""
-        self.prod_value = ""
-
-        if self.database is not None:
-            self.database_value = self.stringify["database"].compile(
-                database=self.database, connection=connection_name
-            )
-            self.value += self.database_value + "."
-
-            self.database_prod_value = self.prod_stringify["database"].compile_prod(
-                database=self.database, connection=connection_name
-            )
-            self.prod_value += self.database_prod_value + "."
-
-        if self.schema is not None:
-            self.schema_value = self.stringify["schema"].compile(
-                table=self.object, schema=self.schema, connection=connection_name
-            )
-            self.value += self.schema_value + "."
-
-            self.schema_prod_value = self.prod_stringify["schema"].compile_prod(
-                table=self.object, schema=self.schema, connection=connection_name
-            )
-            self.prod_value += self.schema_prod_value + "."
-
-        self.object_value = self.stringify["table"].compile(
-            table=self.object, schema=self.schema, connection=connection_name
-        )
-        self.value += self.object_value
-
-        self.object_prod_value = self.prod_stringify["table"].compile_prod(
-            table=self.object, schema=self.schema, connection=connection_name
-        )
-        self.prod_value += self.object_prod_value
-
-    def __hash__(self):
-        return hash(self.get_key())
-
-    def __eq__(self, obj):
-        return self.get_key() == obj.get_key()
-
-    def __lt__(self, obj):
-        return self.get_key() > obj.get_key()
-
-    def __repr__(self):
-        return self.get_key()
-
-    def get_key(self):
-        return f"{self.connection_name}:{self.database or ''}.{self.schema or ''}.{self.object}"
-
-    def get_value(self):
-        return self.value
-
-    def get_prod_value(self):
-        return self.prod_value
-
-
-class DBObjectFactory:
-    def __init__(self, connection_name, stringify, prod_stringify):
-        self.connection_name = connection_name
-        self.stringify = stringify
-        self.prod_stringify = prod_stringify
-
-    def from_components(self, database, schema, object):
-        return DbObject(
-            self.connection_name,
-            database,
-            schema,
-            object,
-            self.stringify,
-            self.prod_stringify,
-        )
-
-    # def from_string(self, obj):
-    #     return DbObject(connection_name, database, schema, obj, self.stringify, self.prod_stringify)
-
 
 class Database:
     """
@@ -248,15 +111,20 @@ class Database:
     """
 
     DDL = DDL
-    DBObjectFactory = DBObjectFactory
 
     def __init__(
-        self, name, name_in_settings, db_type, common_params, stringify, prod_stringify
+        self,
+        name,
+        name_in_settings,
+        db_type,
+        common_params,
+        settings,
     ):
         self.name = name
         self.name_in_settings = name_in_settings
         self.db_type = db_type
         self.max_batch_rows = common_params.get("max_batch_rows", 50000)
+        self._settings = settings
         self._requested_objects = dict()
 
         self._jinja_env = Environment(
@@ -270,10 +138,20 @@ class Database:
             keep_trailing_newline=True,
         )
 
-        self._object_builder = self.DBObjectFactory(
-            self.name, stringify, prod_stringify
+    def _obj_str(
+        self, database: Optional[str], schema: Optional[str], table: str
+    ) -> str:
+        return (
+            f"{database + '.' if database is not None else ''}"
+            f"{schema + '.' if schema is not None else ''}"
+            f"{table}"
         )
 
+    def _fully_qualify(
+        self, database: Optional[str], schema: Optional[str], table: str
+    ) -> str:
+        return self._obj_str(database, schema, table)
+
     def feature(self, feature):
         # Supported sql_features
         #   - CREATE IF NOT EXISTS
@@ -292,9 +170,17 @@ class Database:
         # CANNOT SET SCHEMA
         return feature in ()
 
-    def _set_engine(self, engine):
-        self.engine = engine
-        self.metadata = MetaData(self.engine)
+    def create_engine(self, settings):
+        raise NotImplementedError()
+
+    def _activate_connection(self):
+        self.engine = self.create_engine(self._settings)
+        if self.engine is not None:
+            # We'll have a None engine when the connection is missing from the settings.
+            # We create said object only to allow the config stage to run correctly.
+            self.metadata = MetaData(self.engine)
+            # Force a query to test the connection
+            self.execute("select 1")
 
     def _construct_tests(self, columns, table, schema=None):
         query = """
@@ -313,14 +199,15 @@ class Database:
             for t in tests:
                 breakdown.append(
                     {
-                        "column": col["name"],
+                        "column": col["name"]
+                        if not col["dst_name"]
+                        else col["dst_name"],
                         "type": t["type"],
-                        "status": col[t["type"]],
                         "execute": t["execute"],
                     }
                 )
 
-                if col[t["type"]] is False and t["execute"]:
+                if t["execute"]:
                     count_tests += 1
                     query += template.render(
                         **{
@@ -330,11 +217,12 @@ class Database:
                             if not col["dst_name"]
                             else col["dst_name"],
                             "type": t["type"],
-                            "values": ", ".join(f"'{c}'" for c in t["values"]),
+                            "allowed_values": ", ".join(
+                                f"'{c}'" for c in t["allowed_values"]
+                            ),
                         },
                     )
-            breakdown.append({"print": True})
-        breakdown = breakdown[:-1]
+
         parts = query.splitlines()[:-2]
         query = ""
         for q in parts:
@@ -342,28 +230,32 @@ class Database:
         query += ") AS t;"
 
         if count_tests == 0:
-            return Ok(["", breakdown])
+            return Ok([None, breakdown])
 
         return Ok([query, breakdown])
 
-    def _validate_ddl(self, columns=[], table_properties=[], post_hook=[]):
+    def _validate_ddl(self, columns, table_properties, post_hook):
         if len(columns) == 0 and len(table_properties) == 0 and len(post_hook) == 0:
-            return self._format_properties(self.DDL().get_ddl())
+            properties = self.DDL().get_ddl()
         else:
             try:
-                return self._format_properties(
-                    self.DDL(
+                if table_properties is None or len(table_properties) == 0:
+                    properties = self.DDL(
+                        columns=columns,
+                        post_hook=post_hook,
+                    ).get_ddl()
+                else:
+                    properties = self.DDL(
                         columns=columns,
                         properties=table_properties,
                         post_hook=post_hook,
                     ).get_ddl()
-                )
             except Exception as e:
-
                 return Exc(e, db=self.name, type=self.db_type)
 
-    def _format_properties(self, properties):
+        return self._format_properties(properties)
 
+    def _format_properties(self, properties):
         if properties["columns"]:
             columns = []
             for col in properties["columns"]:
@@ -373,7 +265,7 @@ class Database:
                     "dst_name": col["dst_name"],
                     "unique": False,
                     "not_null": False,
-                    "values": False,
+                    "allowed_values": False,
                 }
                 if "tests" in col:
                     entry.update({"tests": col["tests"]})
@@ -384,12 +276,6 @@ class Database:
 
             properties["columns"] = columns
 
-        if properties["properties"]:
-            for pro in properties["properties"]:
-                for p in pro:
-                    if pro[p] is not None:
-                        properties[p] = pro[p]
-
         return Ok(properties)
 
     def _refresh_metadata(self, only=None, schema=None):
@@ -405,32 +291,38 @@ class Database:
         insp = sqlalchemy.inspect(self.engine)
         out = dict()
 
-        for schema, req_objs in to_introspect.items():
-            if schema == "":
-                schema = None
-            if schema is None:
-                db_objects = [
-                    ("table", insp.get_table_names()),
-                    ("view", insp.get_view_names()),
-                ]
-            else:
-                db_objects = [
-                    ("table", insp.get_table_names(schema)),
-                    ("view", insp.get_view_names(schema)),
-                ]
+        for database, schemas in to_introspect.items():
+            if database != "":
+                # We currently don't support 3 levels of db object specification.
+                raise ValueError("3 level db objects are not currently supported")
 
-            # flatten the results
-            db_objects = {o: t for t, obs in db_objects for o in obs}
+            for schema, req_objs in schemas.items():
+                if schema == "":
+                    schema = None
 
-            if schema not in self._requested_objects:
-                self._requested_objects[schema] = dict()
-
-            out[schema] = dict()
-            for obj_name in req_objs:
-                if obj_name in db_objects:
-                    out[schema][obj_name] = {"type": db_objects[obj_name]}
+                if schema is None:
+                    db_objects = [
+                        ("table", insp.get_table_names()),
+                        ("view", insp.get_view_names()),
+                    ]
                 else:
-                    out[schema][obj_name] = {"type": None}
+                    db_objects = [
+                        ("table", insp.get_table_names(schema)),
+                        ("view", insp.get_view_names(schema)),
+                    ]
+
+                # flatten the results
+                db_objects = {o: t for t, obs in db_objects for o in obs}
+
+                if schema not in self._requested_objects:
+                    self._requested_objects[schema] = dict()
+
+                out[schema] = dict()
+                for obj_name in req_objs:
+                    if obj_name in db_objects:
+                        out[schema][obj_name] = {"type": db_objects[obj_name]}
+                    else:
+                        out[schema][obj_name] = {"type": None}
 
         self._requested_objects = out
 
@@ -554,7 +446,11 @@ class Database:
         batch_size = batch_size or self.max_batch_rows
         buffer = list()
 
-        result = self._validate_ddl(ddl)
+        result = self._validate_ddl(
+            ddl.get("columns", list()),
+            ddl.get("table_properties", dict()),
+            ddl.get("post_hook", list()),
+        )
 
         if result.is_err:
             raise DBError(
