@@ -1,5 +1,5 @@
 from collections import Counter
-from typing import List, Optional, Dict, Union
+from typing import List, Optional, Union
 
 from pydantic import BaseModel, constr, validator, Extra
 from sqlalchemy import create_engine
@@ -9,120 +9,124 @@ from . import Database, Columns, Hook
 
 db_parameters = ["host", "user", "password", "port", "dbname", "cluster_id"]
 
+DistributionStr = constr(regex=r"even|all|key([^,]+)")
 
-class Redshift(Database):
-    class DDL(BaseModel):
-        class Properties(BaseModel):
-            class Sorting(BaseModel):
-                type: Optional[str]
-                columns: List[str]
 
-                class Config:
-                    extra = Extra.forbid
-
-                @validator("type")
-                def validate_type(cls, v, values):
-                    if v.upper() not in ("COMPOUND", "INTERLEAVED"):
-                        raise ValueError(
-                            'Sorting type must be one of "COMPOUND" or "INTERLEAVED"'
-                        )
-
-                    return v.upper()
-
-            distribution: Optional[constr(regex=r"even|all|key([^,]+)")]
-            sorting: Optional[Sorting]
+class DDL(BaseModel):
+    class Properties(BaseModel):
+        class Sorting(BaseModel):
+            type: Optional[str]
+            columns: List[str]
 
             class Config:
                 extra = Extra.forbid
 
-        columns: Optional[List[Columns]] = list()
-        properties: Optional[List[Properties]] = list()
-        post_hook: Optional[List[Hook]] = list()
+            @validator("type")
+            def validate_type(cls, v):
+                if v.upper() not in ("COMPOUND", "INTERLEAVED"):
+                    raise ValueError(
+                        'Sorting type must be one of "COMPOUND" or "INTERLEAVED"'
+                    )
+
+                return v.upper()
+
+        distribution: Optional[DistributionStr]
+        sorting: Optional[Sorting]
 
         class Config:
             extra = Extra.forbid
 
-        @validator("columns", pre=True)
-        def transform_str_cols(cls, v, values):
-            if v is not None and isinstance(v, List):
-                return [{"name": c} if isinstance(c, str) else c for c in v]
-            else:
-                return v
+    columns: List[Union[str, Columns]] = list()
+    properties: Optional[Properties]
+    post_hook: List[Hook] = list()
 
-        @validator("columns")
-        def columns_unique(cls, v, values):
-            dupes = {k for k, v in Counter([e.name for e in v]).items() if v > 1}
-            if len(dupes) > 0:
-                raise ValueError(f"Duplicate columns: {','.join(dupes)}")
-            else:
-                return v
+    class Config:
+        extra = Extra.forbid
 
-        @validator("properties")
-        def validate_distribution(cls, v, values):
-            if len(v) > 0:
-                for props in v:
-                    if props.distribution:
-                        distribution = {
-                            "type": props.upper() if props in ("even", "all") else "KEY"
-                        }
-                        if distribution["type"] == "KEY":
-                            distribution["column"] = props.distribution[
-                                len("key(") : -1
-                            ]
-                            if len(values.get("columns")) > 0 and distribution[
-                                "column"
-                            ] not in values.get("columns"):
-                                raise ValueError(
-                                    f'Distribution key "{distribution["column"]}" is not declared in columns'
-                                )
-                        props.distribution = distribution
-                return v
-            else:
-                return v
+    @validator("columns", pre=True)
+    def transform_str_cols(cls, v):
+        if v is not None and isinstance(v, List):
+            return [{"name": c} if isinstance(c, str) else c for c in v]
+        else:
+            return v
 
-        def get_ddl(self):
-            cols = self.columns
-            self.columns = []
-            for c in cols:
-                tests = []
-                for t in c.tests:
-                    if isinstance(t, str):
-                        tests.append({"type": t, "values": [], "execute": True})
-                    else:
-                        tests.append(
-                            {
-                                "type": t.name if t.name is not None else "values",
-                                "values": t.values if t.values is not None else [],
-                                "execute": t.execute,
-                            }
+    @validator("columns")
+    def columns_unique(cls, v):
+        dupes = {k for k, v in Counter([e.name for e in v]).items() if v > 1}
+        if len(dupes) > 0:
+            raise ValueError(f"Duplicate columns: {','.join(dupes)}")
+        else:
+            return v
+
+    @validator("properties")
+    def validate_distribution(cls, v, values):
+        if v is not None:
+            if v.distribution is not None:
+                distribution = {
+                    "type": v.distribution.upper()
+                    if v.distribution.lower() in ("even", "all")
+                    else "KEY"
+                }
+                if distribution["type"] == "KEY":
+                    distribution["column"] = v.distribution[len("key(") : -1]
+                    if len(values.get("columns")) > 0 and distribution[
+                        "column"
+                    ] not in values.get("columns"):
+                        raise ValueError(
+                            f'Distribution key "{distribution["column"]}" is not declared in columns'
                         )
-                self.columns.append(
-                    {
-                        "name": c.name,
-                        "description": c.description,
-                        "dst_name": c.dst_name,
-                        "type": c.type,
-                        "tests": tests,
-                    }
-                )
+                v.distribution = distribution
 
-            props = self.properties
-            self.properties = []
-            for p in props:
-                self.properties.append(p.dict())
+            return v
+        else:
+            return v
 
-            hook = self.post_hook
-            self.post_hook = []
-            for h in hook:
-                self.post_hook.append(h.dict())
+    def get_ddl(self):
+        columns = list()
+        for c in self.columns:
+            tests = []
+            for t in c.tests:
+                if isinstance(t, str):
+                    tests.append({"type": t, "values": [], "execute": True})
+                else:
+                    tests.append(
+                        {
+                            "type": t.name if t.name is not None else "values",
+                            "values": t.values if t.values is not None else [],
+                            "execute": t.execute,
+                        }
+                    )
+            columns.append(
+                {
+                    "name": c.name,
+                    "description": c.description,
+                    "dst_name": c.dst_name,
+                    "type": c.type,
+                    "tests": tests,
+                }
+            )
 
-            res = {
-                "columns": self.columns,
-                "properties": self.properties,
-                "post_hook": self.post_hook,
-            }
+        res = {
+            "columns": self.columns,
+            "properties": list(),
+            "post_hook": [h.dict() for h in self.post_hook],
+        }
 
-            return res
+        properties = list()
+        if self.properties is not None:
+            if self.properties.distribution is not None:
+                properties.append({"distribution": self.properties.distribution})
+                res["distribution"] = self.properties.distribution
+
+            if self.properties.sorting is not None:
+                properties.append({"sorting": self.properties.sorting})
+                res["sorting"] = self.properties.sorting
+
+        return res
+
+
+class Redshift(Database):
+    DDL = DDL
 
     def feature(self, feature):
         return feature in (
