@@ -3,7 +3,7 @@ SELECT b.tournament_name
      , b.arena_name
      , b.fighter1_name AS fighter_name
      , CASE WHEN b.fighter1_name = b.winner_name THEN 1 ELSE 0 END AS is_winner
-  FROM f_battles b
+  FROM {{ src('f_battles') }} b
 
  UNION
 
@@ -12,4 +12,4 @@ SELECT b.tournament_name
      , b.arena_name
      , b.fighter2_name AS fighter_name
      , CASE WHEN b.fighter2_name = b.winner_name THEN 1 ELSE 0 END AS is_winner
-  FROM f_battles b
+  FROM {{ src('f_battles') }} b
