@@ -1,9 +1,9 @@
 from datetime import datetime
-from typing import Any, Dict, Optional, List
+from typing import Any, Dict, Optional, List, Union
 
 from pydantic import BaseModel, Field, validator, Extra
 from sqlalchemy import or_, select, column
-from terminaltables import AsciiTable
+from colorama import Fore, Style
 
 from ..core.errors import Err, Exc, Ok
 from ..database import Database
@@ -73,7 +73,7 @@ class Config(BaseModel):
     incremental_key: Optional[str]
     max_merge_rows: Optional[int]
     max_batch_rows: Optional[int]
-    columns: Optional[List[Dict[str, Any]]] = list()
+    columns: Optional[List[Union[str, Dict[str, Any]]]] = list()
     table_properties: Optional[List[Dict[str, Any]]] = list()
     post_hook: Optional[List[Dict[str, Any]]] = list()
 
@@ -112,9 +112,6 @@ class CopyTask(SqlTask):
         if "task_name" in self._config_input:
             del self._config_input["task_name"]
 
-        if "columns" in config:
-            self._has_tests = True
-
         conn_names_list = [
             n for n, c in self.connections.items() if isinstance(c, Database)
         ]
@@ -172,43 +169,42 @@ class CopyTask(SqlTask):
 
         # Setup sources
         self.source_db = self.connections[self.task_config.source.db]
-        self.source_schema = self.task_config.source.db_schema
-        self.source_table = self.task_config.source.table
-
-        # Given copy's use case, we should take the table and schema as it comes
-        # TODO add some specification for using `src` so that it works for reverse ETL
-        # if self.source_schema is None:
-        #     self.src(self.source_table, connection=self.source_db)
-        # else:
-        #     self.src(
-        #         f"{self.source_schema}.{self.source_table}", connection=self.source_db
-        #     )
+        if self.task_config.source.db_schema is None:
+            self.source_schema = None
+            self.source_table = self.src(
+                self.task_config.source.table, connection=self.source_db
+            )
+        else:
+            obj = self.src(
+                f"{self.task_config.source.db_schema}.{self.task_config.source.table}",
+                connection=self.source_db,
+            )
+            self.source_schema = obj.split(".")[0]
+            self.source_table = obj.split(".")[1]
 
         # Setup outputs
         self.config_tmp_schema = (
             self.task_config.destination.tmp_schema
             or self.task_config.destination.db_schema
         )
-        self.config_schema = self.task_config.destination.db_schema
-        self.config_table = self.task_config.destination.table
-        self.config_tmp_table = f"sayn_tmp_{self.config_table}"
+        config_schema = self.task_config.destination.db_schema
+        config_table = self.task_config.destination.table
+        config_tmp_table = f"sayn_tmp_{config_table}"
 
-        if self.config_schema is None:
+        if config_schema is None:
             self.schema = None
-            self.table = self.out(self.config_table, connection=self.target_db)
+            self.table = self.out(config_table, connection=self.target_db)
         else:
-            obj = self.out(
-                f"{self.config_schema}.{self.config_table}", connection=self.target_db
-            )
+            obj = self.out(f"{config_schema}.{config_table}", connection=self.target_db)
             self.schema = obj.split(".")[0]
             self.table = obj.split(".")[1]
 
         if self.config_tmp_schema is None:
             self.tmp_schema = None
-            self.tmp_table = self.out(self.config_tmp_table, connection=self.target_db)
+            self.tmp_table = self.out(config_tmp_table, connection=self.target_db)
         else:
             obj = self.out(
-                f"{self.config_schema}.{self.config_tmp_table}",
+                f"{config_schema}.{config_tmp_table}",
                 connection=self.target_db,
             )
             self.tmp_schema = obj.split(".")[0]
@@ -250,7 +246,7 @@ class CopyTask(SqlTask):
         else:
             return result
 
-        if self.run_arguments["command"] == "test":
+        if self.run_arguments["command"] == "test" and len(self.columns["columns"]) > 0:
             result = self.target_db._construct_tests(
                 self.columns["columns"], self.table, self.schema
             )
@@ -260,31 +256,25 @@ class CopyTask(SqlTask):
                 self.test_query = result.value[0]
                 self.test_breakdown = result.value[1]
 
+            if self.test_query is not None:
+                self._has_tests = True
+
         return Ok()
 
-    def setup(self, needs_recompile):
-        if needs_recompile:
-            if self.config_schema is None:
-                self.table = self.out(self.config_table, connection=self.target_db)
-            else:
-                obj = self.out(
-                    f"{self.config_schema}.{self.config_table}",
-                    connection=self.target_db,
-                )
-                self.schema = obj.split(".")[0]
-                self.table = obj.split(".")[1]
-
-            if self.config_tmp_schema is None:
-                self.tmp_table = self.out(
-                    self.config_tmp_table, connection=self.target_db
+    def setup(self):
+        if self.needs_recompile:
+            if self.task_config.source.db_schema:
+                self.source_schema = None
+                self.source_table = self.src(
+                    self.task_config.source.table, connection=self.source_db
                 )
             else:
-                obj = self.out(
-                    f"{self.config_schema}.{self.config_tmp_table}",
-                    connection=self.target_db,
+                obj = self.src(
+                    f"{self.task_config.source.db_schema}.{self.task_config.source.table}",
+                    connection=self.source_db,
                 )
-                self.tmp_schema = obj.split(".")[0]
-                self.tmp_table = obj.split(".")[1]
+                self.source_schema = obj.split(".")[0]
+                self.source_table = obj.split(".")[1]
 
         return Ok()
 
@@ -319,35 +309,95 @@ class CopyTask(SqlTask):
         return result
 
     def test(self):
-        if self.test_query == "":
-            self.info(self.get_test_breakdown(self.test_breakdown))
+        step_queries = {
+            "Write Test Query": self.test_query,
+            "Execute Test Query": self.test_query,
+        }
+        breakdown = self.get_test_breakdown(self.test_breakdown)
+
+        if self.test_query is None:
+            self.info("Nothing to be done")
             return self.success()
         else:
-            with self.step("Write Test Query"):
-                self.write_compilation_output(self.test_query, "test")
+            self.set_run_steps(list(step_queries.keys()))
 
-            with self.step("Execute Test Query"):
-                result = self.default_db.read_data(self.test_query)
+            for step, query in step_queries.items():
+                with self.step(step):
+                    if "Write" in step:
+                        self.write_compilation_output(query, "test")
+                    if "Execute" in step:
+                        try:
+                            result = self.default_db.read_data(query)
+                        except Exception as e:
+                            return Exc(e)
 
-                self.info(self.get_test_breakdown(self.test_breakdown))
+            if len(result) == 0:
+                skipped = [brk for brk in breakdown if brk[0] == "SKIPPED"]
+                executed = [brk for brk in breakdown if brk[0] == "EXECUTED"]
 
-                if len(result) == 0:
-                    return self.success()
-                else:
-                    errout = "Test failed, summary:\n\n"
-                    errout += f"Total number of offending records: {sum([item['cnt'] for item in result])} \n"
-                    data = []
-                    data.append(
-                        ["Breach Count", "Prob. Value", "Test Type", "Failed Fields"]
+                if skipped:
+                    self.info(
+                        f"{Fore.GREEN}{len(skipped)} test(s) {Style.BRIGHT}SKIPPED{Style.NORMAL}"
                     )
+                self.info(
+                    f"{Fore.GREEN}{len(executed)} test(s) {Style.BRIGHT}EXECUTED{Style.NORMAL}"
+                )
 
-                    for res in result:
-                        data.append([res["cnt"], res["val"], res["type"], res["col"]])
-                    table = AsciiTable(data)
+                return self.success()
+            else:
+                skipped = []
+                executed = []
+                failed = []
+                for brk in breakdown:
+                    if any(brk[1] != res["type"] for res in result) or any(
+                        brk[2] != res["col"] for res in result
+                    ):
+                        if brk[0] == "SKIPPED":
+                            skipped.append(brk)
+                        if brk[0] == "EXECUTED":
+                            executed.append(brk)
+                    else:
+                        failed.append(brk)
+                if self.run_arguments["debug"]:
 
-                    errinfo = f"You can find the compiled test query at compile/{self.group}/{self.name}_test.sql"
+                    fl_info = [f"{Fore.RED}FAILED: "]
+                    for info in failed:
+                        count = sum(
+                            [
+                                item["cnt"]
+                                for item in result
+                                if (item["type"] == info[1] and item["col"] == info[2])
+                            ]
+                        )
+                        values = [
+                            item["val"]
+                            for item in result
+                            if (item["type"] == info[1] and item["col"] == info[2])
+                        ]
+                        values = ", ".join(values[:5])
+                        fl_info.append(
+                            f"{Fore.RED}{Style.BRIGHT}{brk[1]} test{Style.NORMAL} on {Style.BRIGHT}{info[2]} FAILED{Style.NORMAL}. {count} offending records. \n\t    Please see some values for which the test failed: {Style.BRIGHT}{values}{Style.NORMAL}"
+                        )
+                    if skipped:
+                        self.info(
+                            f"{Fore.GREEN}{len(skipped)} test(s) {Style.BRIGHT}SKIPPED{Style.NORMAL}"
+                        )
+                    self.info(
+                        f"{Fore.GREEN}{len(executed)} test(s) {Style.BRIGHT}EXECUTED{Style.NORMAL}"
+                    )
+                    for err in fl_info:
+                        self.info(err)
 
-                    return self.fail(errout + table.table + "\n\n" + errinfo)
+                    errinfo = f"Test Failed. You can find the compiled test query at compile/{self.group}/{self.name}_test.sql"
+                    return self.fail(errinfo)
+                else:
+                    summary = f"{len(executed)} tests were ran, {len(executed)-len(failed)} succeeded, "
+                    if skipped:
+                        summary += f", {len(skipped)} were skipped, "
+                    summary += f"{len(failed)} failed."
+                    self.warning(summary)
+                    errout = ", ".join(list(set([res["type"] for res in result])))
+                    return self.fail(f"Failed test types: {errout}")
 
     def execute(self, execute, debug, is_full_load, limit=None):
         # Introspect target
