@@ -1,7 +1,10 @@
-from . import Task
+from .task import Task
 
 
 class DummyTask(Task):
+    def config(self):
+        return self.success()
+
     def setup(self):
         return self.success()
 
@@ -10,3 +13,6 @@ class DummyTask(Task):
 
     def run(self):
         return self.success()
+
+    def test(self):
+        return self.success()
